#include <vector>
#include <string>
#include <tuple>
#include <iostream>
#include <random>
#include <chrono>

using std::cout;

// Count the minimum number of single Nucleotide edits require to change one Sequence into the other.
// http://en.wikipedia.org/wiki/Levenshtein_distance
int LevenshteinDistance(const std::string& s1, const std::string& s2) {
    // degenerate cases
    if (s1 == s2) return 0;
    if (s1.size() == 0) return s2.size();
    if (s2.size() == 0) return s1.size();

    // create two work vectors of integer distances
    std::vector<int> v0(s2.size() + 1);
    std::vector<int> v1(s2.size() + 1);

    // initialize v0 (the previous row of distances)
    // this row is A[0][i]: edit distance for an empty s
    // the distance is just the number of characters to delete from t
    for (size_t i=0; i < v0.size(); i++) {
        v0[i] = int(i);
    }

    for (int i=0; i < s1.size(); i++) {
        // calculate v1 (current row distances) from the previous row v0

        // first element of v1 is A[i+1][0]
        //   edit distance is delete (i+1) chars from s to match empty t
        v1[0] = i+1;

        // use formula to fill in the rest of the row
        for (int j=0; j < s2.size(); j++)         {
            const int cost = (s1[i] == s2[j]) ? 0 : 1;
            v1[j+1] = std::min(v1[j]+1, std::min(v0[j+1]+1, v0[j]+cost));
        }

        //  swap v0 with v1
        std::swap(v0, v1);
    }


    return v1[s2.size()];
}

auto calc_vev(const std::vector<std::string>& strings) {
    std::vector<int> dists(strings.size(), std::numeric_limits<int>::max());
    std::vector<int> idxs(strings.size(), -1);

    for(int i=0; i < strings.size(); ++i) {
        for(int j=0; j < strings.size(); ++j) {
            if(j==i) {
                continue;
            }

            int d = LevenshteinDistance(strings[i], strings[j]);
            if(d < dists[i]) {
                dists[i] = d;
                idxs[i] = j;
            }
        }
    }

    return std::pair(dists, idxs);
}

int main() {
    // 1000 Strings a 200 Zeichen mit ATCG wuerfel und gegeneinander teste
    std::vector<std::string> strings(1000);


    std::random_device r;
    std::default_random_engine e1(r());
    std::uniform_int_distribution<int> uniform_dist(0, 3);

    for(std::string& s : strings) {
        s.reserve(200);
        for(int i=0; i < 200; i++) {
            int x = uniform_dist(e1);
            switch(x) {
            case 0: s.push_back('A'); break;
            case 1: s.push_back('T'); break;
            case 2: s.push_back('C'); break;
            case 3: s.push_back('G'); break;
            default: throw "Uuhuuhh";
            }
        }
    }

    cout << strings.size() << " random strings created. levs...\n";
    auto start = std::chrono::system_clock::now();
    const auto result = calc_vev(strings);
    cout <<  std::chrono::duration<double>(std::chrono::system_clock::now() - start).count() << " sec\n";
    for(int dist : result.first) {
        cout << dist << " ";
    }
    cout << "\n";
}
